
var CombineLayer = cc.Layer.extend({
    dragon_tar:null,
    array_part:null,
    array_listeners:null,
    index:0,
    labelName:null,
    dragonName:null,
    curStep:null,
    ctor:function (index) {
        //////////////////////////////
        // 1. super init first
        this._super();

        /////////////////////////////
        // 2. add a menu item with "X" image, which is clicked to quit the program
        //    you may modify it.
        // ask the window size
        var size = cc.winSize;
        this.curStep = 0;
        this.array_part = [null, null, null, null];
        this.array_listeners = [null, null, null, null];
        this.index = index;
        this.dragonName = Config_Common.array_dinosaurs[index].did;

        this.createComponents();

        addBackMenu(this.menu, this, function(){
            cc.director.popScene();
        });

        return true;
    },

    createComponents:function () {
        var size = cc.winSize;

        var scale = size.height == 1001 ? 1 : 0.8;

        var background = new cc.Sprite(resPath("figure", "background"));
        background.setAnchorPoint(cc.p(0, 0));
        background.setPosition(cc.p(0, 0));
        background.scaleX = winWidth() / 1334;
        this.addChild(background, 1);

        this.dragon_tar = new DragonCombineItem(this.dragonName);
        this.dragon_tar.setPosition(Config_Figure.tar_pos.begin);
        this.addChild(this.dragon_tar, 5);

        var array_part = [1,2,3,4];
        array_part = shuffle(array_part);
        for (var n_index = 0; n_index < 4; ++n_index){
            this.array_part[n_index] = new DragonPartItem(this.dragonName, array_part[n_index]);
            this.array_part[n_index].setPosition(Config_Figure.array_pos_src[n_index]);
            this.array_part[n_index].setScale(0.6);
            this.array_part[n_index].setOpacity(0);
            this.addChild(this.array_part[n_index], 10 + n_index);
        }

        this.labelName = new cc.LabelTTF(Config_Common.array_dinosaurs[this.index].name, "Arial", 75);
        this.labelName.setPosition(Config_Figure.pos_name);
        this.labelName.setFontFillColor(cc.color(254, 87, 54, 255));
        this.labelName.setOpacity(0);
        this.addChild(this.labelName, 54);
        //
        this.addListeners();
        this.reshuffleGame();
    },
    onEnter:function(){
        this._super();
    },
    onExitTransitionDidStart:function(){
        this._super();
    },
    addListeners:function(){
        var self = this;
        var sprite_listener = cc.EventListener.create({
            event: cc.EventListener.TOUCH_ONE_BY_ONE,
            swallowTouches: true,
            onTouchBegan: function (touch, event) {
                var target = event.getCurrentTarget();
                var locationInNode = target.convertToNodeSpace(touch.getLocation());

                var s = target.getContentSize();
                var rect = cc.rect(0, 0, s.width, s.height);

                if (cc.rectContainsPoint(rect, locationInNode) && !target.isChecked()) {
                    self.dragon_tar.showTips(target.getIndex());
                    return true;
                }

                return false;
            },
            onTouchMoved: function (touch, event) {
                var target = event.getCurrentTarget();
                var delta = touch.getDelta();
                target.x += delta.x;
                target.y += delta.y;
            },
            onTouchEnded: function (touch, event) {
                var target = event.getCurrentTarget();
                self.checkTarget(target);
                cc.log("on touch end!!");
            },
            onTouchCanceled: function (touch, event) {
                cc.log("on touch canceled!!!");
            }
        });

        cc.eventManager.addListener(sprite_listener, this.array_part[0]);
        cc.eventManager.addListener(sprite_listener.clone(), this.array_part[1]);
        cc.eventManager.addListener(sprite_listener.clone(), this.array_part[2]);
        cc.eventManager.addListener(sprite_listener.clone(), this.array_part[3]);
    },
    pauseAllListeners:function () {
        for (var n_index = 0; n_index < 4; ++n_index){
            cc.eventManager.pauseTarget(this.array_part[n_index], true);
        }
    },
    resumeAllListeners:function () {
        for (var n_index = 0; n_index < 4; ++n_index){
            cc.eventManager.resumeTarget(this.array_part[n_index], true);
        }
    },
    checkTarget:function(target){
        this.pauseAllListeners();
        var point = this.dragon_tar.partPoint(target.getIndex())

        var dis = distance(target.getPosition(), point);

        if (dis < Config_Combine.sucDis){
            console.log("success");
            playAudio(audioPath("right"), false, false);
            target.setChecked(true);
            this.curStep++;
            target.runAction(cc.sequence(
                cc.moveTo(0.1, point),
                cc.callFunc(function () {
                    target.setOpacity(0);
                    var index = indexOfObject(this.array_part, target);
                    if (index != -1){
                        target.setPosition(Config_Figure.array_pos_src[index]);
                    }
                    this.dragon_tar.setSucPart(target.getIndex());
                }.bind(this)),
                cc.delayTime(0.5),
                cc.callFunc(function () {
                    this.resumeAllListeners();
                    if (this.curStep >= 4){
                        cc.sys.localStorage.setItem(this.dragonName, 1);
                        this.index = (this.index + 1) % 10;
                        this.scheduleOnce(function () {
                            cc.director.runScene(new CombineScene(this.index));
                        }.bind(this), 1);
                    }
                }.bind(this))
            ));

        } else {
            this.dragon_tar.restorePart(target.getIndex());
            playAudio(audioPath("wrong"), false, false);
            var index = indexOfObject(this.array_part, target);
            if (index != -1){
                target.runAction(cc.sequence(
                    cc.moveTo(0.2, Config_Figure.array_pos_src[index]),
                    cc.callFunc(function () {
                        this.resumeAllListeners();
                    }.bind(this))
                ));
            }

        }

    },
    reshuffleGame:function(){
        this.dragon_tar.runAction(cc.moveTo(0.5, Config_Figure.tar_pos.mid));
        this.labelName.setOpacity(0);
        if ("jia" == Config_Common.array_dinosaurs[this.index].did) {
            this.labelName.setPosition(pAdd(Config_Figure.pos_name, cc.p(-130, 0)));
        }else {
            this.labelName.setPosition(Config_Figure.pos_name);
        }
        this.labelName.setString(Config_Common.array_dinosaurs[this.index].name);
        this.scheduleOnce(function () {
            this.labelName.runAction(cc.sequence(
                cc.fadeIn(0.1),
                cc.callFunc(function () {
                    playAudio(audioPath(Config_Common.array_dinosaurs[this.index].did), false, false);
                }.bind(this)),
                cc.scaleTo(0.1, 1.1),
                cc.scaleTo(0.2, 0.9),
                cc.scaleTo(0.1, 1)
            ));

            for (var n_itor = 0; n_itor < this.array_part.length; ++n_itor){
                this.array_part[n_itor].runAction(cc.fadeIn(0.2));
            }
        }.bind(this), 0.6);
    },
    genDragonArray:function(){
        var array = [];
        var array_src = [];
        if (0 == this.index){
            array = Config_Common.array_dinosaurs.slice(1, 9);
            array = shuffle(array);
            array_src[0] = Config_Common.array_dinosaurs[0];
            array_src = array_src.concat(array.slice(0, 3));
            return shuffle(array_src);
        }else if((Config_Common.array_dinosaurs.length - 1) == this.index){
            array = Config_Common.array_dinosaurs.slice(0, 9);
            array = shuffle(array);
            array_src[0] = Config_Common.array_dinosaurs[9];
            array_src = array_src.concat(array.slice(0, 3));
            return shuffle(array_src);
        }else{
            array = Config_Common.array_dinosaurs.slice(0, this.index - 1);
            array = array.concat(Config_Common.array_dinosaurs.slice(this.index + 1, Config_Common.array_dinosaurs.length - 1));
            array = shuffle(array);
            array_src[0] = Config_Common.array_dinosaurs[this.index];
            array_src = array_src.concat(array.slice(0,3));
            return shuffle(array_src);
        }
    },
    cleanup:function () {
        this._super();


    }
});

var CombineScene = cc.Scene.extend({
    ctor:function (index) {
        this._super();
        CB_CALLED = false;
        var layer = new CombineLayer(index);
        this.addChild(layer);
    }
});

